#include "comparison.h"

int compare_ip(struct in_addr ip1,struct in_addr ip2) {
	if (ntohl(ip1.s_addr)>ntohl(ip2.s_addr)){
	    return 1;
	}
	if (ntohl(ip1.s_addr)<ntohl(ip2.s_addr)){
	    return -1;
	}
	return 0;
}

int compare_mask(struct in_addr mask1,struct in_addr mask2) {
    int i;
    
    for (i=31;i>=0;i--){
	    if ((mask1.s_addr&(0x80000000>>i)) > (mask2.s_addr&(0x80000000>>i)))
		return 1;
	    if ((mask1.s_addr&(0x80000000>>i)) < (mask2.s_addr&(0x80000000>>i)))
		return -1;
	}
    return 0;
}

int compare_iface(uint16_t in1,uint16_t in2) {
    if (in1>in2)
	return 1;
    if (in1<in2)
	return -1;
    return 0;
}

int compare_eth(unsigned char * p1, unsigned char *p2) {
    int i;
    
    for (i=0;i<ETHER_ADDR_LEN;i++) {
	if (p1[i]>p2[i])
	    return 1;
        if (p1[i]<p2[i]) 
	    return -1;
    }
    return 0;
}

int comparison(Ppointer p1, Ppointer p2) {
    in_addr_t mask1,mask2;
    int i;
    if ((i=compare_mask((*p1)->netmask.sin.sin_addr,(*p2)->netmask.sin.sin_addr)) !=0)
	    return i;
	/*mask equal */
    if ((i=compare_ip((*p1)->dst.sin.sin_addr,(*p2)->dst.sin.sin_addr)) != 0)
	    return i;
    if (weight[(*p1)->type] < weight[(*p2)->type])
	return -1;
    if (weight[(*p1)->type] > weight[(*p2)->type])
	return 1;
    /* all different types have already been resolved */
    switch ((*p1)->type){
    case (ROUTE_INET):
	/* here we have all sockaddr_in addresses */
	/*only gate can be different*/
	return (compare_ip((*p1)->gateway.sin.sin_addr,(*p2)->gateway.sin.sin_addr));
    case (ROUTE_IFACE):
	/*only gate can be different*/
	return (compare_iface((*p1)->gateway.sdl.sdl_index,(*p2)->gateway.sdl.sdl_index));
    case (ROUTE_ETH):
	return (compare_eth((unsigned char *)LLADDR(&((*p1)->gateway.sdl)),(unsigned char *)LLADDR(&((*p2)->gateway.sdl))));
    case (ROUTE_LAST): default:
	err_msg("Bad type value");
	(*p1)->not_allowed=1;
	(*p2)->not_allowed=1;
    }
    return 0;
}

int comparison_dst_mask(Ppointer p1, Ppointer p2) {
    in_addr_t mask1,mask2;
    int i;

    if ((i=compare_mask((*p1)->netmask.sin.sin_addr,(*p2)->netmask.sin.sin_addr)) !=0)
	    return i;
	/*mask equal */
    if ((i=compare_ip((*p1)->dst.sin.sin_addr,(*p2)->dst.sin.sin_addr)) != 0)
	    return i;
    return 0;
}

