#include "conrt.h"
#include <mife.h>
#include <babolo/parser.h>
#include <babolo/BLINflag.h>

int comparison(Ppointer p1, Ppointer p2);

int instr_err(u_char *from, int length) {
    u_char *instr_err;

    if ((instr_err = (u_char *)malloc(sizeof(u_char)*length))==NULL) {
        syslog(LOG_ALERT,"Malloc Error\n");
        exit(-1);
    }
    strncpy(instr_err,from,length-1);
    instr_err[length-1]='\0';
    err_msg("Input string error \"%s\"",instr_err);
    free(instr_err);
}

int t_ip_mask(char *word,struct sockaddr_in *ip,struct sockaddr_in *mask) {
    char *q, *r,*w;
    int i,j;
    int masklen=0;
    struct in_addr *pin;
    
    pin = (struct in_addr *)Malloc(sizeof(struct in_addr));
    q=r=word;
    i=strlen(word);
    while (r < q + i) {
	if (*r=='/') {
	    *r='\0';
	    if (!inet_aton(q,pin)) {
		*r='/';
		free(pin);
	        return -1;
		
	    }
	    else {
		*r='/';
		ip->sin_family=AF_INET;
		ip->sin_addr.s_addr=pin->s_addr;
		ip->sin_len = sizeof(struct sockaddr_in);
		free(pin);
		for (j=r-q+1;j<i;j++) {
		    if (*(q+j)>='0' &&  *(q+j)<='9')
			masklen=masklen*10+(*(q+j)-'0');
		    else 
			return -1;
		}
	        if (masklen==0) {
		    mask->sin_addr.s_addr=htonl(0x00000000);
		}
                else if (masklen>0 && masklen<33) {
		    mask->sin_addr.s_addr=htonl(((0xffffffff)<<(32-masklen)));
		}
		else {
		    return -1;
		}
		mask->sin_len= sizeof(struct sockaddr_in);
		mask->sin_family=0;
		ip->sin_addr.s_addr&=mask->sin_addr.s_addr;
		return 0;
	    }
	}
	r++;
    }
    return -1;
}

int t_ip(char *word,struct sockaddr_in *ip) {
    int i,j;
    struct in_addr *pin;

    pin = (struct in_addr *)Malloc(sizeof(struct in_addr));
    if (!inet_aton(word,pin)) {
	free(pin);
	return -1;
    }
    else {
	ip->sin_family=AF_INET;
	ip->sin_addr.s_addr=pin->s_addr;
	ip->sin_len = sizeof(struct sockaddr_in);
	free(pin);
	return 0;
    }
    return -1;
}

int t_if(const char *name, struct sockaddr_dl *sdl){
    struct ifaddrs *ifap=NULL,*ifa=NULL;
    struct sockaddr_dl *sdl2=NULL;
	
    if (getifaddrs(&ifap)) {
	err_msg("getifaddrs failed");
	return -1;
    }
    for (ifa = ifap; ifa; ifa = ifa->ifa_next) {
	if (ifa->ifa_addr->sa_family != AF_LINK)
	    continue;
	if (strcmp(name, ifa->ifa_name))
	    continue;
	sdl2 = (struct sockaddr_dl *)ifa->ifa_addr;
    }
    if (sdl2) {
        memcpy(sdl, sdl2, sdl2->sdl_len);
        freeifaddrs(ifap);
	return 0;
    }
    return -1;
}

int t_eth(const char *eth_addr, struct sockaddr_dl *sdl){
    /*Actually we can't prepare the whole address */
    struct ether_addr   *ea;

    sdl->sdl_len=sizeof(struct sockaddr_dl);
    sdl->sdl_family = AF_LINK;
    ea = (struct ether_addr *)LLADDR(sdl);
    if (Ether_aton(eth_addr, ea) == 0){
        sdl->sdl_alen = ETHER_ADDR_LEN;
	return 0;
    }
    else
	return -1;
}

int addTask(Rpointer *Task, Ppointer *Index){
    int TaskN=0;
    struct mife_descriptor *in;
    int tasklen;
    u_char *q;
    u_char *r;
    u_char *r_old;
    struct babolo_parm *bp;
    PAddress paddr_d,paddr_g,paddr_n;
    int MaxRoute = MAXROUTE;
    int legal;
    int i,k;

    /* Preparing the Input Stream */
    if (!(in=mife_opef(MIFE_FULL, fileno(stdin)))) 
	err_quit("Reading task failed");
    if ((tasklen = mife_read(in, 0, 0, NULL)) <= 0 || !(r = q = mife_get(in, 0))) 
	err_quit("MIFE error reading");

    while (r < q + tasklen){
	r_old=r;
	bp = babolo_getparm(Bpars_NOEN | Bpars_CRLN | Bpars_LFLN, &r, "");
	legal=0;
	if  (!bp) {
	    err_quit("Input string error : \"%s\"", r);
        }
	else {
	    paddr_d = &((*Task)[TaskN].dst);
	    paddr_g = &((*Task)[TaskN].gateway);
	    paddr_n = &((*Task)[TaskN].netmask);
	    bzero(paddr_d,sizeof(Address));
	    bzero(paddr_g,sizeof(Address));
	    bzero(paddr_n,sizeof(Address));
	    switch (strtoul(bp->argv[0],NULL,10)) {
	    case ROUTE_INET:
		if (bp->argc !=3) {
		    legal=0;
		}
		else if (!t_ip_mask(bp->argv[1],&(paddr_d->sin),&(paddr_n->sin)) && !t_ip(bp->argv[2],&(paddr_g->sin))) {
		    (*Task)[TaskN].type=ROUTE_INET;
		    legal=1;
		}
		break;
	    case ROUTE_IFACE:
		if (bp->argc !=3) {
		    legal=0;
		}
		else if (!t_ip_mask(bp->argv[1],&(paddr_d->sin),&(paddr_n->sin)) && !t_if(bp->argv[2],&(paddr_g->sdl))) {
		    (*Task)[TaskN].type=ROUTE_IFACE;
		    legal=1;
		}
		else
		    legal=0;
		break;
	    case ROUTE_ETH:
		if (bp->argc !=3) {
		    instr_err(r_old,r-r_old);
		}
		else if (!t_ip(bp->argv[1],(struct sockaddr_in *)&(paddr_d->sinarp)) && !t_eth(bp->argv[2],&(paddr_g->sdl))) {
		    (*Task)[TaskN].type=ROUTE_ETH;
		    paddr_n->sin.sin_addr.s_addr=0xffffffff;
		    paddr_n->sin.sin_family=0;
		    paddr_n->sin.sin_len=sizeof(struct sockaddr_in);
		    legal=1;
		}
		else
		    legal=0;
		break;
	    default:
		legal=0;
		break;
	    }
	    if (legal) {
		(*Task)[TaskN].not_allowed=0;	 
		TaskN++;
		if (TaskN>MaxRoute-1) {
                    MaxRoute=MaxRoute*2;
                    (*Task)=(Rpointer)Reallocf((*Task),sizeof(Route)*MaxRoute);
		}
	    }
	    else
		instr_err(r_old,r-r_old);
	}
	babolo_freeparm(bp);
    }
    (*Task) = (Rpointer)Reallocf((*Task),sizeof(Route)*TaskN);
    (*Index) = (Ppointer)Reallocf((*Index),sizeof(Rpointer)*TaskN);
    for (i=0;i<TaskN;i++) {
        (*Index)[i]=&((*Task)[i]);
    }
    qsort((void **) (*Index), TaskN, sizeof(Rpointer), (int (*)(const void *, const void *))comparison);
    return (TaskN);
}